#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
 #include <avr/power.h> // Required for 16 MHz Adafruit Trinket
#endif

#include "Wire.h"
#include "I2Cdev.h"
#include "MPU6050_6Axis_MotionApps20.h"

MPU6050 mpu;
#define INTERRUPT_PIN 2  // use pin 2 on Arduino Uno & most boards
#define PIN        6 
#define NUMPIXELS 16 //Set the correct number of LEDs

Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

uint32_t activeColor, oldActiveColor=0;

bool dmpReady = false;
uint8_t mpuIntStatus;
uint8_t devStatus;
uint16_t packetSize;
uint16_t fifoCount;
uint8_t fifoBuffer[64];

Quaternion q;           
VectorFloat gravity;    
float rotace[3];        
int x,y,z;

volatile bool mpuInterrupt = false;

void dmpDataReady() {
    mpuInterrupt = true;
}

void setup() {
  
  Serial.begin(115200);
  pixels.begin();
  pixels.clear();
  pixels.setBrightness (128);
  
  #if defined(__AVR_ATtiny85__) && (F_CPU == 16000000)
    clock_prescale_set(clock_div_1);
  #endif
  // join I2C bus (I2Cdev library doesn't do this automatically)
  #if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
      Wire.begin();
      Wire.setClock(400000); // 400kHz I2C clock. Comment this line if having compilation difficulties
  #elif I2CDEV_IMPLEMENTATION == I2CDEV_BUILTIN_FASTWIRE
      Fastwire::setup(400, true);
  #endif
  
  while (!Serial);
  
  Serial.println(F("Initializing I2C devices..."));
  mpu.initialize();
  pinMode(INTERRUPT_PIN, INPUT);
  // verify connection
  Serial.println(F("Testing device connections..."));
  Serial.println(mpu.testConnection() ? F("MPU6050 connection successful") : F("MPU6050 connection failed"));
  // wait for ready
//  Serial.println(F("\nSend any character to begin DMP programming and demo: "));
//    while (Serial.available() && Serial.read()); // empty buffer
//    while (!Serial.available());                 // wait for data
//    while (Serial.available() && Serial.read()); // empty buffer again
  // load and configure the DMP
  Serial.println(F("Initializing DMP..."));
  devStatus = mpu.dmpInitialize();
   // supply your own gyro offsets here, scaled for min sensitivity
  mpu.setXGyroOffset(0);
  mpu.setYGyroOffset(0);
  mpu.setZGyroOffset(0);
  mpu.setZAccelOffset(1688); // 1688 factory default for my test chip
     // make sure it worked (returns 0 if so)
  if (devStatus == 0) {
      // Calibration Time: generate offsets and calibrate our MPU6050
        mpu.CalibrateAccel(6);
        mpu.CalibrateGyro(6);
        mpu.PrintActiveOffsets();
        // turn on the DMP, now that it's ready
        Serial.println(F("Enabling DMP..."));
        mpu.setDMPEnabled(true);

        // enable Arduino interrupt detection
        Serial.print(F("Enabling interrupt detection (Arduino external interrupt "));
        Serial.print(digitalPinToInterrupt(INTERRUPT_PIN));
        Serial.println(F(")..."));
        attachInterrupt(digitalPinToInterrupt(INTERRUPT_PIN), dmpDataReady, RISING);
        mpuIntStatus = mpu.getIntStatus();

        // set our DMP Ready flag so the main loop() function knows it's okay to use it
        Serial.println(F("DMP ready! Waiting for first interrupt..."));
        dmpReady = true;

        // get expected DMP packet size for later comparison
        packetSize = mpu.dmpGetFIFOPacketSize();
    } else {
        // ERROR!
        // 1 = initial memory load failed
        // 2 = DMP configuration updates failed
        // (if it's going to break, usually the code will be 1)
        Serial.print(F("DMP Initialization failed (code "));
        Serial.print(devStatus);
        Serial.println(F(")"));
  }
}

void loop() {
 if (!dmpReady) return;
 if (mpu.dmpGetCurrentFIFOPacket(fifoBuffer)) { // Get the Latest packet 

            // display Euler angles in degrees
            mpu.dmpGetQuaternion(&q, fifoBuffer);
            mpu.dmpGetGravity(&gravity, &q);
            mpu.dmpGetYawPitchRoll(rotace, &q, &gravity);
   }

    Serial.print("X ");
    Serial.print(rotace[2] * 180/M_PI);
    Serial.print(" \t Y ");
    Serial.print(rotace[1] * 180/M_PI);
    Serial.print(" \t Z ");
    Serial.println(rotace[0] * 180/M_PI);

    x=rotace[2] * 180/M_PI;
    y=rotace[1] * 180/M_PI;
    z=rotace[0] * 180/M_PI;

   if(abs(x)<45 && abs(y)<45){
      activeColor=pixels.Color( 255,255,255); //White 
    }else
    if( x>45 && abs(x)<135 && (abs(y)<45|| abs(y)>135)){
      activeColor=pixels.Color(  255, 0, 0);  //Red when turn to side
    }else
    if(x<-45 && abs(x)<135 && (abs(y)<45|| abs(y)>135)){
      activeColor=pixels.Color( 0, 255,  0);  //Green when turn to the second side
    }else
    if(y>45 && abs(y)<135 && (abs(x)<45|| abs(x)>135)){
      activeColor=pixels.Color( 255, 255,  0);  //Yellow when turn to the third side
    }else
    if(y<-45 && abs(y)<135 && (abs(x)<45 || abs(x)>135)){
      activeColor=pixels.Color( 0,0 ,  255);   //Blue when turn to the fourth side
    }else
    if(abs(y)>135 && abs(x)>135){
      activeColor=pixels.Color( 0,0 ,  0); // Black when upside down
    }
    if(activeColor != oldActiveColor){
      pixels.clear();
      pixels.fill(activeColor);
      pixels.show();
      oldActiveColor=activeColor;
    }   
}
